--[[
	Title: 		oUF_Saul
	Version: 	40000.9
	Requires: 	oUF 1.5.x
	Credits: 	Haste for oUF and oUF_Lily
]]--

local localized, class = UnitClass('player')
local font = "Fonts\\ARIALN.ttf"
local fontsize = 11
local fontsizesmall = 9

local TEXTURE = [[Interface\AddOns\oUF_Saul\textures\minimalist]]
local COMBOTEX = [[Interface\AddOns\oUF_Saul\textures\checkmark]]
local BACKDROP = {
	bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=],
	insets = {top = -1, left = -1, bottom = -1, right = -1},
}
local SHADOWS = {
	edgeFile = "Interface\\AddOns\\oUF_Saul\\textures\\glowTex", 
	edgeSize = 4,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}

oUF.colors.class['WARLOCK'] = {117/255, 82/255, 221/255} -- Just to make the warlock class color and soul shard colors match

oUF.colors.power = {
	["MANA"] = {26/255, 139/255, 255/255},
	["RAGE"] = {255/255, 26/255, 48/255},
	["FOCUS"] = {80/255, 185/255, 230/255},
	["ENERGY"] = {255/255, 225/255, 26/255},
	["HAPPINESS"] = {0.00, 1.00, 1.00},
	["RUNES"] = {0.50, 0.50, 0.50},
	["RUNIC_POWER"] = {0.00, 0.82, 1.00},
	["AMMOSLOT"] = {0.80, 0.60, 0.00},
	["FUEL"] = {0.0, 0.55, 0.5},
	["HOLY_POWER"] = {0.96, 0.55, 0.73},
	["SOUL_SHARDS"] = {117/255, 82/255, 221/255},
}

oUF.colors.happiness = {
	[1] = {182/225, 34/255, 32/255},
	[2] = {220/225, 180/225, 52/225},
	[3] = {143/255, 194/255, 32/255},
}

oUF.colors.reaction = {
	[1] = {182/255, 34/255, 32/255},
	[2] = {182/255, 34/255, 32/255},
	[3] = {182/255, 92/255, 32/255},
	[4] = {220/225, 180/255, 52/255},
	[5] = {143/255, 194/255, 32/255},
	[6] = {143/255, 194/255, 32/255},
	[7] = {143/255, 194/255, 32/255},
	[8] = {143/255, 194/255, 32/255},
}
		
oUF.colors.runes = {
	{196/255, 30/255, 58/255};
	{173/255, 217/255, 25/255};
	{35/255, 127/255, 255/255};
	{178/255, 53/255, 240/255};
}

local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if(cunit == 'Vehicle') then
		cunit = 'Pet'
	end

	if(unit == "party") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local function hex(r, g, b)
	if(type(r) == 'table') then
		if(r.r) then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
	end
	return string.format('|cff%02x%02x%02x', r * 255, g * 255, b * 255)
end

local function truncate(value)
	if(value >= 1e6) then
		return gsub(format('%.1fm', (value / 1e6)), '^%.?0+([km])$', '%1')
	elseif(value >= 1e3) then
		return gsub(format('%.0fk', (value / 1e3)), '^%.?0+([km])$', '%1')
	else
		return value
	end
end

local function ColourGradient(perc)
	if perc <= 0.5 then
		return 255, perc*510, 0
	else
		return 510 - perc*510, 255, 0
	end
end


--[[
 	tags
]]--

oUF.Tags['name'] = function(unit)

	local c = (not UnitIsConnected(unit)) and oUF.colors.disconnected or
		(not UnitIsPlayer(unit)) and oUF.colors.reaction[UnitReaction(unit, 'player')] or
		(UnitFactionGroup(unit) and UnitIsEnemy(unit, 'player') and UnitIsPVP(unit)) and {1, 0, 0} or {1, 1, 1}
		
	return format('|cff%02x%02x%02x%s|r', c[1] * 255, c[2] * 255, c[3] * 255, UnitName(unit))
end
oUF.TagEvents['name'] = 'UNIT_NAME_UPDATE UNIT_REACTION UNIT_FACTION'

oUF.Tags['difficulty']  = function(unit) 
	local level = UnitLevel(unit); 
	return UnitCanAttack('player', unit) and hex(GetQuestDifficultyColor((level > 0) and level or 99)) or '|cff0090ff'
end

oUF.TagEvents['hp'] = 'UNIT_HEALTH UNIT_MAXHEALTH'
oUF.Tags['hp'] = function(unit)
	local min, max = UnitHealth(unit), UnitHealthMax(unit)
	local r,g,b = ColourGradient(min/max)
	
	if (min == 0) then
		return 'DEAD'
	else
		return format('|cff%02x%02x%02x%s|r', r, g, b, truncate(min))
	end
end

oUF.TagEvents['hpprec'] = 'UNIT_HEALTH UNIT_MAXHEALTH'
oUF.Tags['hpperc'] = function(unit)
	local min, max = UnitHealth(unit), UnitHealthMax(unit)
	if (min ~= 0) then
		return format('%s%%', floor(min/max*100))
	end
end

oUF.TagEvents['pp'] = oUF.TagEvents['curpp']
oUF.Tags['pp'] = function(unit)
	local power = UnitPower(unit)
	
	if (power > 0) then
		local num, str = UnitPowerType(unit)
		local c = oUF.colors.power[str]
		return c and format('|cff%02x%02x%02x%s|r', c[1] * 255, c[2] * 255, c[3] * 255, truncate(power))
	end
end
	
--[[
 	functions
]]--
local function updateCombo(self, event, unit)
	if(unit == PlayerFrame.unit and unit ~= self.CPoints.unit) then
		self.CPoints.unit = unit
	end
end

local PostUpdateHealth = function(Health, unit, min, max)
	if(UnitIsDead(unit) or UnitIsGhost(unit)) then
		Health:SetValue(0)
	end
end

local PostCreateIcon = function(Auras, button)

	button.cd.noCooldownCount = false	-- hide CDC CDs
	Auras.disableCooldown = false		-- hide CD spiral
	Auras.showDebuffType = true			-- show debuff border type color 
	
	local count = button.count
	count:ClearAllPoints()
	count:SetPoint"BOTTOMRIGHT"

    button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
    button.icon:SetDrawLayer("BACKGROUND")
    
    button.overlay:SetTexture([=[Interface\AddOns\oUF_saul\textures\iconborder]=])
    button.overlay:SetPoint("TOPLEFT", button, "TOPLEFT", -1, 1)
    button.overlay:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 1, -1)
    button.overlay:SetTexCoord(0.03, 0.97, 0.03, 0.97)
    button.overlay.Hide = function(self) self:SetVertexColor(0, 0, 0) end

end

local PostUpdateIcon
do
	local playerUnits = {
		player = true,
		pet = true,
		vehicle = true,
	}
	PostUpdateIcon = function(icons, unit, icon, index, offset, filter, isDebuff)
		local texture = icon.icon
		if(playerUnits[icon.owner]) then
			texture:SetDesaturated(false)
		else
			texture:SetDesaturated(true)
		end
	end
end

local RAID_TARGET_UPDATE = function(self, event)
	local index = GetRaidTargetIndex(self.unit)
	if(index) then
		self.RIcon:SetText(ICON_LIST[index].."22|t")
	else
		self.RIcon:SetText()
	end
end

-- custom castbar text
local function CustomTimeText(self, duration)
	self.Time:SetFormattedText('%.1f / %.1f', duration, self.max)
end


--[[
 	layout
]]--
local Shared = function(self, unit, isSingle)

	self.menu = menu
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks"AnyDown"
	
	self:SetBackdrop({
		  bgFile =  [=[Interface\ChatFrame\ChatFrameBackground]=],
		  edgeFile = "Interface\\Buttons\\WHITE8x8", 
		  tile = false, tileSize = 0, edgeSize = 1, 
		  insets = { left = -1, right = -1, top = -1, bottom = -1}
		})
	self:SetBackdropColor(.1,.1,.1,1)
	self:SetBackdropBorderColor(.3,.3,.3,1)	
		
	local shadow = CreateFrame("Frame", nil, self)
	shadow:SetFrameLevel(1)
	shadow:SetFrameStrata(self:GetFrameStrata())
	shadow:SetPoint("TOPLEFT", -4, 4)
	shadow:SetPoint("BOTTOMRIGHT", 4, -4)
	shadow:SetBackdrop(SHADOWS)
	shadow:SetBackdropColor(0, 0, 0, 0)
	shadow:SetBackdropBorderColor(0, 0, 0, 1)
	
	local Health = CreateFrame("StatusBar", nil, self)
	Health:SetStatusBarTexture(TEXTURE)
	Health:SetStatusBarColor(.2,.2,.2,1)
	Health:SetPoint('TOPRIGHT', self, -2, -2)
	Health:SetPoint('TOPLEFT', self, 2, -2)
	Health.frequentUpdates = true
	self.Health = Health

	local HealthBG = Health:CreateTexture(nil, "BORDER")
	HealthBG:SetAllPoints(self.Health)
	HealthBG:SetTexture(.3,.3,.3)
	Health.bg = HealthBG

    local Power = CreateFrame("StatusBar", nil, self)
	Power:SetHeight(8)
	Power:SetStatusBarTexture(TEXTURE)
	Power:SetParent(self)
	Power:SetPoint('BOTTOMRIGHT', self, -2, 2)
	Power:SetPoint('BOTTOMLEFT', self, 2, 2)
	self.Power = Power
	
	Power.bg = Power:CreateTexture(nil, 'BORDER')
	Power.bg:SetAllPoints(Power)
	Power.bg:SetTexture(.3,.3,.3)
	Power.bg.multiplier = 0.5
		
	Power.frequentUpdates = true
	Power.colorTapping = true
	Power.colorHappiness = true
	Power.colorClass = true
	Power.colorReaction = true
	Power.colorDisconnected = true
       
	Health.PostUpdate = PostUpdateHealth
		
	local Leader = self:CreateTexture(nil, "OVERLAY")
	Leader:SetSize(16, 16)
	Leader:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
	self.Leader = Leader

	local MasterLooter = self:CreateTexture(nil, 'OVERLAY')
	MasterLooter:SetSize(16, 16)
	MasterLooter:SetPoint('TOPLEFT', self, 'TOPLEFT', 20, 0)
	self.MasterLooter = MasterLooter

	local RaidIcon = Health:CreateFontString(nil, "OVERLAY")
	RaidIcon:SetPoint("TOPRIGHT", 4, 4)
	RaidIcon:SetFontObject(GameFontNormalSmall)
	self.RIcon = RaidIcon
	self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
	table.insert(self.__elements, RAID_TARGET_UPDATE)

	local info = self.Health:CreateFontString(nil, 'OVERLAY')
	info:SetFont(font, fontsize)
	info:SetHeight(18)
	info:SetJustifyH('LEFT')
	info.frequentUpdates = 0.1
	
	local value = self.Health:CreateFontString(nil, 'OVERLAY')
	value:SetFont(font, fontsize)
	value:SetHeight(18)
	value:SetJustifyH('RIGHT')
	value.frequentUpdates = 0.1

	if(unit == 'player' or unit == 'target' or unit == 'party') then
	
		self:SetSize(230, 36)
		Health:SetHeight(23)
		
		info:SetPoint('LEFT', self.Health, 2, 0)
		info:SetWidth(180)
    	self:Tag(info, '[difficulty][smartlevel] [rare]|r[name]')
		
		value:SetPoint('RIGHT', self.Health, -2, 0)
		value:SetWidth(100)
		self:Tag(value, '[hp] [hpperc] [pp]')

		local cb = CreateFrame("StatusBar", nil, self)
		cb:SetStatusBarTexture(TEXTURE)
		cb:SetStatusBarColor(.2,.2,.2,.8)
		cb:SetAllPoints(Power)
		cb:SetToplevel(true)
		self.Castbar = cb
	
		cb.Text = cb:CreateFontString(nil, 'OVERLAY')
		cb.Text:SetFont(font, fontsizesmall)
		cb.Text:SetHeight(18)
		cb.Text:SetJustifyH('LEFT')
		cb.Text:SetPoint('LEFT', cb, 4, 0)
		
		cb.Time = cb:CreateFontString(nil, 'OVERLAY')
		cb.Time:SetFont(font, fontsizesmall)
		cb.Time:SetHeight(18)
		cb.Time:SetJustifyH('RIGHT')		
		cb.Time:SetPoint("RIGHT", cb, -4, 0)
		cb.CustomTimeText = CustomTimeText
		
	elseif(unit == 'pet') then
	
		self:SetSize(130, 36)
		Health:SetHeight(23)
		
		info:SetPoint('LEFT', self.Health, 2, 0)
		info:SetWidth(98)
    	self:Tag(info, '[name]')
		
		value:SetPoint('RIGHT', self.Health, -2, -0)
		self:Tag(value, '[hpperc]')

	elseif(unit == 'focus' or unit == 'targettarget') then

	    self:SetSize(130, 22)
     	Health:SetHeight(18)
     	Power:Hide()

		info:SetPoint('LEFT', self.Health, 2, 0)
		info:SetWidth(98)
    	self:Tag(info, '[name]')
    	
		value:SetPoint('RIGHT', self.Health, -2, 0)
		self:Tag(value, '[hpperc]')

	elseif(unit == 'raid') then
		
		self:SetSize(85, 26)
		Health:SetHeight(18)
		Power:SetHeight(3)
		info:SetPoint('LEFT', self.Health, 2, 0)
		info:SetWidth(50)		
		self:Tag(info, '[name]')
    	
		value:SetPoint('RIGHT', self.Health, -2, 0)
		self:Tag(value, '[hpperc]')
	
	end

	-- enable range checking
	if(unit == 'party' or unit == 'raid') then
		self.Range = {	insideAlpha = 1, outsideAlpha = .5,	}
	end
	
	if(unit == 'target' and (class == 'ROGUE' or class == 'DRUID')) then
		local cpframe = CreateFrame('Frame', nil, self)
		cpframe:SetFrameStrata("HIGH")

		self.CPoints = {}
		self.CPoints.unit = "player"
		self.CPoints[1] = cpframe:CreateTexture(nil, "OVERLAY")
		self.CPoints[1]:SetHeight(14)
		self.CPoints[1]:SetWidth(14)
		self.CPoints[1]:SetPoint("LEFT", Health, "TOPLEFT", 4, 3)
		self.CPoints[1]:SetTexture(COMBOTEX)

		for i=2, 5 do
			self.CPoints[i] = cpframe:CreateTexture(nil, "OVERLAY")
			self.CPoints[i]:SetHeight(14)
			self.CPoints[i]:SetWidth(14)
			self.CPoints[i]:SetPoint("LEFT", self.CPoints[i-1], "RIGHT", 1)
			self.CPoints[i]:SetTexture(COMBOTEX)
		end
		self.CPoints[5]:SetVertexColor(1, 0, 0)
	end

	if(class == 'DEATHKNIGHT' and unit == 'player') then
		self.Runes = CreateFrame('Frame', nil, self)
		
		for i = 1, 6 do
			self.Runes[i] = CreateFrame("StatusBar", self:GetName().."_Runes"..i, self)
			self.Runes[i]:SetHeight(10)
			self.Runes[i]:SetWidth(24)
			self.Runes[i]:SetStatusBarTexture(TEXTURE)
			self.Runes[i]:SetBackdrop(BACKDROP)
			self.Runes[i]:SetBackdropColor(.1,.1,.1,1)				
			
			self.Runes[i].bg = self.Runes[i]:CreateTexture(nil, "BORDER")
			self.Runes[i].bg:SetAllPoints(self.Runes[i])				
			self.Runes[i].bg:SetTexture(0.3, 0.3, 0.3)
				
			if (i == 1) then
				self.Runes[i]:SetPoint("BOTTOMLEFT", UIParent, "CENTER", -87, -130)
			else
				self.Runes[i]:SetPoint("TOPLEFT", self.Runes[i-1], "TOPRIGHT", 6, 0)
			end	
		end
	end

	if(class == "WARLOCK" and unit == 'player') then
    	local color = self.colors.power["SOUL_SHARDS"]
		self.SoulShards = CreateFrame("Frame", nil, self)
			
		for i = 1, 3 do
			self.SoulShards[i] = CreateFrame("StatusBar", self:GetName().."_Shards"..i, self)
			self.SoulShards[i]:SetHeight(10)
			self.SoulShards[i]:SetWidth(24)
			self.SoulShards[i]:SetStatusBarTexture(TEXTURE)
            self.SoulShards[i]:SetStatusBarColor(color[1], color[2], color[3])	
			self.SoulShards[i]:SetBackdrop(BACKDROP)
			self.SoulShards[i]:SetBackdropColor(.1,.1,.1,1)	
				
			if (i == 1) then
				self.SoulShards[i]:SetPoint("BOTTOMLEFT", UIParent, "CENTER", -42, -130)
			else                                                     
				self.SoulShards[i]:SetPoint('TOPLEFT', self.SoulShards[i-1], "TOPRIGHT", 6, 0)
			end	
		end
	end
	
end

local UnitSpecific = {

	target = function(self, ...)
		Shared(self, ...)

		local Buffs = CreateFrame("Frame", nil, self)
		Buffs.initialAnchor = 'TOPLEFT'
		Buffs["growth-x"] = 'RIGHT'
		Buffs['growth-y'] = 'DOWN'
  		Buffs:SetPoint('TOPLEFT', self, 'TOPRIGHT', 4, -1)
		Buffs:SetHeight(123)
		Buffs:SetWidth(123)
		Buffs.num = 25
		Buffs.size = 21
        Buffs.spacing = 4
		self.Buffs = Buffs

		local Debuffs = CreateFrame("Frame", nil, self)
  		Debuffs:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 1, -4)
		Debuffs.showDebuffType = true
		Debuffs.initialAnchor = 'TOPLEFT'
		Debuffs:SetHeight(75)
		Debuffs:SetWidth(230)
		Debuffs.num = 27
		Debuffs.size = 21
		Debuffs.spacing = 4
		Debuffs['growth-x'] = 'RIGHT'
		Debuffs['growth-y'] = 'DOWN'
		self.Debuffs = Debuffs

		Debuffs.PostCreateIcon = PostCreateIcon
		Debuffs.PostUpdateIcon = PostUpdateIcon
		Buffs.PostCreateIcon = PostCreateIcon
		Buffs.PostUpdateIcon = PostUpdateIcon
	end,

	targettarget = function(self, ...)
		Shared(self, ...)

		local Health, Power = self.Health, self.Power
		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetPoint('BOTTOMLEFT', self, 'TOPLEFT', 1, 4)
		Auras.showDebuffType = true
		Auras:SetWidth(132)
		Auras:SetHeight(26)
		Auras.size = 21
		Auras.spacing = 6
		Auras.gap = false
		Auras.numBuffs = 0
		Auras.numDebuffs = 5

		Auras.PostCreateIcon = PostCreateIcon
		self.Auras = Auras
	end,

	focus = function(self, ...)
		Shared(self, ...)

		local Health, Power = self.Health, self.Power
		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 1, 1)
		Auras.showDebuffType = true
		Auras:SetWidth(132)
		Auras:SetHeight(26)
		Auras.size = 21
		Auras.spacing = 6
		Auras.gap = false
		Auras.numBuffs = 0
		Auras.numDebuffs = 5

		Auras.PostCreateIcon = PostCreateIcon
		self.Auras = Auras
	end,
	
	party = function(self, ...)
		Shared(self, ...)

		local Health, Power = self.Health, self.Power
		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 1, -4)
		Auras.showDebuffType = true
		Auras:SetWidth(230)
		Auras:SetHeight(21)
		Auras.size = 21
		Auras.spacing = 4
		Auras.gap = true
		Auras.numBuffs = 4
		Auras.numDebuffs = 3
		Auras.PostCreateIcon = PostCreateIcon
		self.Auras = Auras
	end,
}


--[[
 	register style
]]--
oUF:RegisterStyle("Saul", Shared)
oUF:RegisterStyle("SaulRaid", Shared)

for unit,layout in next, UnitSpecific do
	oUF:RegisterStyle('Saul - ' .. unit:gsub("^%l", string.upper), layout)
end

local spawnHelper = function(self, unit, ...)
	if(UnitSpecific[unit]) then
		self:SetActiveStyle('Saul - ' .. unit:gsub("^%l", string.upper))
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	else
		self:SetActiveStyle'Saul'
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	end
end


--[[
 	factory
]]--

oUF:Factory(function(self)
	spawnHelper(self, 'player', 'CENTER', -250, -220)
	spawnHelper(self, 'target', 'CENTER', 250, -220)
	spawnHelper(self, 'targettarget', 'CENTER', 0, -214)
	spawnHelper(self, 'focus', 'CENTER', 0, -242)
	spawnHelper(self, 'pet', 'CENTER', -442, -220)

	self:SetActiveStyle('Saul - Party')
	local party = self:SpawnHeader(
		nil, nil, 'party,solo',
		'showParty', true, 
		'showPlayer', false, 
		'showSolo', false, 
		'yOffset', -30
	)
	party:SetPoint("TOPLEFT", 20, -180)

	self:SetActiveStyle('SaulRaid')
	local raid = oUF:SpawnHeader("oUF_Raid", nil, "solo,party,raid", 
		"showRaid", true,  
		"showPlayer", true,
		"showSolo", false,
		"showParty", false,
		"xoffset", 0,
		"yOffset", -3,
		"groupFilter", "1,2,3,4,5,6,7,8",
		"groupingOrder", "1,2,3,4,5,6,7,8",
		"groupBy", "GROUP",
		"maxColumns", 5,
		"unitsPerColumn", 5,
		"columnSpacing", 3,
		"point", "TOP",
		"columnAnchorPoint", "LEFT",
		"oUF-initialConfigFunction", ([[
		self:SetWidth(%d)
		self:SetHeight(%d)
	]]):format(85, 26))
	raid:SetPoint("TOPLEFT", 10, -10)
		
end)